#include "font.h"

void 
font_glyph_metrics_init(Font* self, GlyphMetrics* glyphMetrics, char glyph) 
{ 
	TTF_GlyphMetrics
	( 
		self->ttf, 
		glyph, 
		&glyphMetrics->minX, 
		&glyphMetrics->maxX, 
		&glyphMetrics->minY, 
		&glyphMetrics->maxY, 
		&glyphMetrics->advance 
	); 
}

s32 
font_glyph_width_get(Font* self, char glyph) 
{ 
	GlyphMetrics glyphMetrics; 

	font_glyph_metrics_init(self, &glyphMetrics, glyph); 

	return glyphMetrics.maxX - glyphMetrics.minX;
} 

s32 
font_glyph_kerning_get(Font* self, char glyph, char nextGlyph) 
{ 
	return TTF_GetFontKerningSizeGlyphs(self->ttf, glyph, nextGlyph); 
} 

void 
font_glyph_texture_init(Font* self, Texture* texture, char glyph) 
{ 
	SDL_Surface* surface; 
	SDL_Surface* textSurface; 
	ivec2 size; 

	size[0] = font_glyph_width_get(self, glyph); 

	size[1] = font_height_get(self); 

	surface_rgba_init(&surface, size); 

	textSurface = TTF_RenderGlyph_Blended(self->ttf, glyph, GLYPH_COLOR); 
	
	SDL_BlitSurface(textSurface, NULL, surface, NULL); 

	SDL_FreeSurface(textSurface); 

	texture_surface_init(texture, surface); 

	SDL_FreeSurface(surface); 
}

s32
font_line_skip_get(Font* self)
{
	return TTF_FontLineSkip(self->ttf);
}

s32
font_height_get(Font* self)
{
	return TTF_FontHeight(self->ttf);
}

bool
font_init(Font* self, const char* path, u32 size)
{
	self->ttf = TTF_OpenFont(path, size);

	if (!self->ttf)
	{
		printf(STRING_SDL_TTF_FONT_ERROR, path, TTF_GetError());
		return false;
	}

	printf(STRING_SDL_TTF_FONT_SUCCESS, path);

	self->size = size;
	self->isInit = true;

	for (s32 i = 0; i < FONT_GLYPH_MAX; i++)
		font_glyph_texture_init(self, &self->glyphTextures[i], (char)i);

	return true;
}

void
font_free(Font* self)
{
	if (self->isInit)
	{
		TTF_CloseFont(self->ttf);
		
		for (s32 i = 0; i < FONT_GLYPH_MAX; i++)
			texture_free(&self->glyphTextures[i]);

		self->isInit = false;
	}
}
