#include "mouse.h"

void
mouse_update(Mouse* self)
{
        s32 state;
	s32 x;
	s32 y;

        memcpy(&self->previous, &self->current, sizeof(bool) * MOUSE_BUTTON_COUNT);
        memset(&self->current, '\0', sizeof(bool) * MOUSE_BUTTON_COUNT);

        state = SDL_GetMouseState(NULL, NULL);

        if ((state & SDL_BUTTON_LMASK) != 0)
                self->current[MOUSE_BUTTON_LEFT] = true;

        if ((state & SDL_BUTTON_MMASK) != 0)
                self->current[MOUSE_BUTTON_MIDDLE] = true;

        if ((state & SDL_BUTTON_RMASK) != 0)
                self->current[MOUSE_BUTTON_RIGHT] = true;

        if ((state & SDL_BUTTON_X1) != 0)
                self->current[MOUSE_BUTTON_X1] = true;

        if ((state & SDL_BUTTON_X2) != 0)
                self->current[MOUSE_BUTTON_X2] = true;

	SDL_GetMouseState(&x, &y);

	self->position[0] = x;
	self->position[1] = y;
}

bool
mouse_press(Mouse* self, MouseButtonType type)
{
        return (self->current[type] && !self->previous[type]);
}

bool
mouse_held(Mouse* self, MouseButtonType type)
{
        return (self->current[type] && self->previous[type]);
}

bool
mouse_release(Mouse* self, MouseButtonType type)
{
        return (!self->current[type] && self->previous[type]);
}

void
mouse_buffer_position_get(Mouse* self, RendererBuffer buffer, Renderer* renderer, vec2 position)
{
	vec2 mousePosition;

	mousePosition[0] = (f32)self->position[0];
	mousePosition[1] = (f32)self->position[1];

	renderer_buffer_position_from_window_position(renderer, buffer, mousePosition, position);
}

f32
mouse_angle_from_buffer_position_get(Mouse* self, RendererBuffer buffer, Renderer* renderer, vec2 position)
{
	vec2 mousePosition;
	f32 angle;

	mouse_buffer_position_get(self, buffer, renderer, mousePosition);

	return ATAN(position[0], mousePosition[0], position[1], mousePosition[1]);
}
