#include "component_animation.h"

void _component_animation_frame_set(ComponentAnimation* self, ECS* ecs);

void
_component_animation_frame_set(ComponentAnimation* self, ECS* ecs)
{
	ComponentTimer* timer;

	if (self->isFinished)
		return;

	timer = ecs_component_get(ecs, COMPONENT_TIMER, self->timer);

	if (timer->isFinished)
	{
		ComponentAtlas* atlas;

		if (self->frame >= self->animation.count)
		{
			if (!self->animation.isLoop)
			{
				self->isFinished = true;
				return;
			}

			self->frame = 0;
		}

		atlas = ecs_component_get(ecs, COMPONENT_ATLAS, self->component.id);

		atlas->index = self->animation.frameIndices[self->frame];

		component_timer_init(timer, ecs, self->animation.length);

		self->frame++;
	}
}

void
component_animation_add(ComponentAnimation* self, ECS* ecs)
{
	ComponentTimer* timer;
	
	self->timer = entity_timer_add(ecs, -1);

	timer = ecs_component_get(ecs, COMPONENT_TIMER, self->timer);

	timer->component.isDisabled = true;
}

void
component_animation_delete(ComponentAnimation* self, ECS* ecs)
{
	ecs_entity_delete(ecs, self->timer);
}

void
component_animation_set(ComponentAnimation* self, ECS* ecs, Animation* animation)
{
	ComponentTimer* timer;

	timer = ecs_component_get(ecs, COMPONENT_TIMER, self->timer);

	memcpy(&self->animation, animation, sizeof(Animation));

	self->frame = 0;
	self->isFinished = false;

	component_timer_init(timer, ecs, self->animation.length);
	timer->component.isDisabled = false;
	timer->isFinished = true;

	_component_animation_frame_set(self, ecs);
}

void
component_animation_init(ComponentAnimation* self, ECS* ecs, Animation* animation)
{
	component_animation_set(self, ecs, animation);
}

void
component_animation_tick(ComponentAnimation* self, ECS* ecs)
{
	_component_animation_frame_set(self, ecs);
}
