#include "component_collectible.h"

void _component_collectible_increment(ComponentCollectible* self, ECS* ecs, EntityID id);
void _component_collectible_collect(ComponentCollectible* self, ECS* ecs, EntityID id);

void
_component_collectible_increment(ComponentCollectible* self, ECS* ecs, EntityID id)
{
	ComponentFood* food;
	ComponentPower* power;
	ComponentStamina* stamina;
	ComponentColorChange* colorChange;

	switch (self->type)
	{
		case COLLECTIBLE_FOOD:
			food = ecs_component_get(ecs, COMPONENT_FOOD, id);
			if (!food)
				return;
			food->value += COMPONENT_COLLECTIBLE_FOOD_VALUE * food->multiplier;
			sound_play(&ecs->resources->sounds[SOUND_FOOD_COLLECT]);
			break;
		case COLLECTIBLE_POWER:
			colorChange = ecs_component_get(ecs, COMPONENT_COLOR_CHANGE, id);
			power = ecs_component_get(ecs, COMPONENT_POWER, id);
			if (!power)
				return;
			power->value += COMPONENT_COLLECTIBLE_POWER_VALUE;
			sound_play(&ecs->resources->sounds[SOUND_POWER]);

			if (!colorChange)
				return;

			component_color_change_init(colorChange, ecs, COMPONENT_COLLECTIBLE_POWER_COLOR,TRANSPARENT, COMPONENT_COLLECTIBLE_POWER_COLOR_CHANGE_TIME, true, false);
			break;
		case COLLECTIBLE_STAMINA:
			stamina = ecs_component_get(ecs, COMPONENT_STAMINA, id);
			colorChange = ecs_component_get(ecs, COMPONENT_COLOR_CHANGE, id);
			if (!stamina)
				return;
			stamina->value += COMPONENT_COLLECTIBLE_STAMINA_VALUE;
			sound_play(&ecs->resources->sounds[SOUND_STAMINA]);

			if (!colorChange)
				return;

			component_color_change_init(colorChange, ecs, COMPONENT_COLLECTIBLE_STAMINA_COLOR,TRANSPARENT, COMPONENT_COLLECTIBLE_STAMINA_COLOR_CHANGE_TIME, true, false);
			break;
		case COLLECTIBLE_ROTTEN_FOOD:
			food = ecs_component_get(ecs, COMPONENT_FOOD, id);
			if (!food)
				return;
			food->value += COMPONENT_COLLECTIBLE_ROTTEN_FOOD_VALUE;

			sound_play(&ecs->resources->sounds[SOUND_BLEH]);
			break;
		case COLLECTIBLE_AMULET:
			sound_play(&ecs->resources->sounds[SOUND_POWER]);
			break;
		default:
			break;
	}
}

void
_component_collectible_collect(ComponentCollectible* self, ECS* ecs, EntityID id)
{
	_component_collectible_increment(self, ecs, id);
	ecs_entity_delete(ecs, self->component.id);
}

void
component_collectible_add(ComponentCollectible* self, ECS* ecs)
{
	ecs_components_add
	(
		ecs, 
		COMPONENT_COLLECTIBLE_DEPENDENCIES, 
		COMPONENT_COLLECTIBLE_DEPENDENCY_COUNT, 
		self->component.id
	);
}

void
component_collectible_init
(
	ComponentCollectible* self, 
	ECS* ecs,
	Texture* texture, 
	const vec2 size, 
	const vec3 position,
	CollectibleType type
)
{
	component_world_object_init
	(
		ecs_component_get(ecs, COMPONENT_WORLD_OBJECT, self->component.id),
		ecs,
		texture,
		size,
		position,
		COMPONENT_COLLECTIBLE_HEIGHT_OFFSET,
		COMPONENT_COLLECTIBLE_RADIUS,
		true
	);

	component_levitate_init
	(
		ecs_component_get(ecs, COMPONENT_LEVITATE, self->component.id),
		ecs,
		COMPONENT_COLLECTIBLE_LEVITATE_DISTANCE,
		COMPONENT_COLLECTIBLE_LEVITATE_FREQUENCY
	);

	self->type = type;
}

void
component_collectible_tick(ComponentCollectible* self, ECS* ecs)
{
	ComponentCircleCollider* circleCollider;
	ComponentHoming* homing;
	bool isTarget;
	bool isAbleToBeCollected;
	EntityID target;

	circleCollider = ecs_component_get(ecs, COMPONENT_CIRCLE_COLLIDER, self->component.id);
	homing = ecs_component_get(ecs, COMPONENT_HOMING, self->component.id);

	isTarget = false;
	isAbleToBeCollected = true;

	for (s32 i = 0; i < (s32)ecs->lists[COMPONENT_COLLECT].components.count; i++)
	{
		ComponentCollect* collect;
		
		collect = ecs_component_from_index_get(ecs, COMPONENT_COLLECT, i);

		if (circle_collide_check(&collect->circle, &circleCollider->circle))
		{
			target = collect->component.id;
			isTarget = true;
		}
	}

	if (!isTarget)
		return;

	if (self->type == COLLECTIBLE_FOOD)
	{
		ComponentFood* food;

		food = ecs_component_get(ecs, COMPONENT_FOOD, target);

		if (food->value >= food->max)
		{
			homing->isHoming = false;
			isAbleToBeCollected = false;
			//sound_play(&ecs->resources->sounds[SOUND_CANT_CARRY]);
		}
	}

	if (isAbleToBeCollected)
	{
		if (!homing->isHoming)
			component_homing_init(homing, ecs, target, COMPONENT_COLLECTIBLE_HOMING_SPEED);
		
		if (component_circle_collider_collide_check(circleCollider, ecs, target))
			_component_collectible_collect(self, ecs, target);
	}
}
