#include "component_color_change.h"

void
component_color_change_add(ComponentColorChange* self, ECS* ecs)
{
	self->timerID = entity_timer_add(ecs, -1);

	self->component.isDisabled = true;
}

void
component_color_change_delete(ComponentColorChange* self, ECS* ecs)
{
	ecs_entity_delete(ecs, self->timerID);
}

void
component_color_change_tick(ComponentColorChange* self, ECS* ecs)
{
	ComponentTextureQuad* textureQuad;
	ComponentText* text;
	ComponentTimer* timer;
	vec4 colorStep;
	vec4* color;
	vec4* colorCover;

	if (self->isFinished)
		return;

	timer = ecs_component_get(ecs, COMPONENT_TIMER, self->timerID);

	ecs->isLog = false;

	textureQuad = ecs_component_get(ecs, COMPONENT_TEXTURE_QUAD, self->component.id);
	text = ecs_component_get(ecs, COMPONENT_TEXT, self->component.id);

	ecs->isLog = true;

	if (textureQuad)
	{
		color = &textureQuad->color;
		colorCover = &textureQuad->colorCover;
	}
	else if (text)
	{
		color = &text->color;
		colorCover = &text->colorCover;
	}
	else
		return;

	if (self->isColorCover)
		glm_vec4_scale(self->colorStep, timer->value, *colorCover); 
	else
		glm_vec4_scale(self->colorStep, timer->value, *color); 

	if (timer->isFinished)
	{
		self->isFinished = true;

		if (self->isReturn)
		{
			if (self->isColorCover)
				glm_vec4_copy(self->returnColor, *colorCover);
			else
				glm_vec4_copy(self->returnColor, *color);
		}
		return;
	}
}

void
component_color_change_init(ComponentColorChange* self, ECS* ecs, const vec4 endColor, const vec4 beginColor, u32 length, bool isColorCover, bool isReturn)
{
	ComponentTimer* timer;
	ComponentTextureQuad* textureQuad;
	ComponentText* text;
	vec4 colorDifference;

	ecs->isLog = false;

	textureQuad = ecs_component_get(ecs, COMPONENT_TEXTURE_QUAD, self->component.id);
	text = ecs_component_get(ecs, COMPONENT_TEXT, self->component.id);

	ecs->isLog = true;

	glm_vec4_copy((f32*)beginColor, self->beginColor);
	glm_vec4_copy((f32*)endColor, self->endColor);

	if (textureQuad)
		glm_vec4_copy((f32*)textureQuad->color, self->returnColor);
	else if (text)
		glm_vec4_copy((f32*)text->color, self->returnColor);

	timer = ecs_component_get(ecs, COMPONENT_TIMER, self->timerID);

	component_timer_init(timer, ecs, length);

	timer->component.isDisabled = false;

	glm_vec4_sub(self->endColor, self->beginColor, colorDifference);
	glm_vec4_scale(colorDifference, (1.0f / length), self->colorStep);

	self->isColorCover = isColorCover;
	self->isReturn = isReturn;
	self->isFinished = false;

	self->component.isDisabled = false;
}
