#include "component_drop_shadow.h"

static void _component_drop_shadow_texture_quad_set(ComponentDropShadow* self, ECS* ecs);

static void
_component_drop_shadow_texture_quad_set(ComponentDropShadow* self, ECS* ecs)
{
	ComponentTextureQuad* textureQuadDropShadow;
	ComponentTextureQuad* textureQuad;

	textureQuad = ecs_component_get(ecs, COMPONENT_TEXTURE_QUAD, self->component.id);

	if (!self->isSpriteInit)
	{
		self->spriteID = entity_sprite_add
		(
			ecs,
			textureQuad->texture,
			textureQuad->buffer,
			textureQuad->origin,
			textureQuad->size,
			textureQuad->position
		);

		self->isSpriteInit = true;
	}

	textureQuadDropShadow = ecs_component_get(ecs, COMPONENT_TEXTURE_QUAD, self->spriteID);
	
	memcpy(textureQuadDropShadow, textureQuad, sizeof(ComponentTextureQuad));

	textureQuadDropShadow->component.id = self->spriteID;
	glm_vec4_copy(self->color, textureQuadDropShadow->color);

	textureQuadDropShadow->position[0] += self->offset[0];
	textureQuadDropShadow->position[1] += self->offset[1];
	textureQuadDropShadow->position[2] += COMPONENT_DROP_SHADOW_Z_OFFSET;
}

void
component_drop_shadow_add(ComponentDropShadow* self, ECS* ecs)
{
	self->spriteID = ecs_entity_add(ecs);
}

void
component_drop_shadow_delete(ComponentDropShadow* self, ECS* ecs)
{
	ecs_entity_delete(ecs, self->spriteID);
}

void
component_drop_shadow_tick(ComponentDropShadow* self, ECS* ecs)
{
	_component_drop_shadow_texture_quad_set(self, ecs);
}

void
component_drop_shadow_init(ComponentDropShadow* self, ECS* ecs, vec4 color, vec2 offset)
{
	glm_vec4_copy(color, self->color);
	glm_vec2_copy(offset, self->offset);
	
	_component_drop_shadow_texture_quad_set(self, ecs);
}


