#include "entity_player.h"

EntityID
entity_player_add(ECS* ecs, const vec3 position)
{
	EntityID id;
	ComponentWorldObject* worldObject;
	ComponentColorChange* colorChange;

	id = ecs_entity_add(ecs);

	ecs_components_add(ecs, ENTITY_PLAYER_DEPENDENCIES, ENTITY_PLAYER_DEPENDENCY_COUNT, id);
	
	worldObject = ecs_component_get(ecs, COMPONENT_WORLD_OBJECT, id), 

	component_world_object_init
	(
	 	worldObject,
		ecs, 
		&ecs->resources->textures[TEXTURE_PLAYER], 
		PLAYER_SIZE, 
		position,
		PLAYER_HEIGHT_OFFSET,
		PLAYER_RADIUS,
		true
	);
	
	worldObject->angle = PLAYER_ANGLE;

	component_atlas_init
	(
		ecs_component_get(ecs, COMPONENT_ATLAS, id),
		ecs,
		PLAYER_ATLAS_FRAME_SIZE,
		PLAYER_ATLAS_SIZE,
		PLAYER_ATLAS_INDEX
	);

	component_stamina_init
	(
		ecs_component_get(ecs, COMPONENT_STAMINA, id),
		ecs,
		PLAYER_STAMINA_MAX,
		PLAYER_STAMINA_GAIN
	);

	component_power_init
	(
		ecs_component_get(ecs, COMPONENT_POWER, id),
		ecs,
		PLAYER_POWER_MAX,
		PLAYER_POWER_GAIN
	);

	component_collect_init
	(
		ecs_component_get(ecs, COMPONENT_COLLECT, id),
		ecs,
		PLAYER_COLLECT_RADIUS
	);

	component_food_init
	(
		ecs_component_get(ecs, COMPONENT_FOOD, id),
		ecs,
		0,
		PLAYER_FOOD_MAX,
		PLAYER_FOOD_MULTIPLIER
	);

	colorChange = ecs_component_get(ecs, COMPONENT_COLOR_CHANGE, id);

	colorChange->component.isDisabled = true;

	return id;
}
