#include "cutscene.h"

static void _cutscene_dialogue_set(Cutscene* self);
static void _cutscene_sprite_set(Cutscene* self);

static void 
_cutscene_dialogue_set(Cutscene* self)
{
	ComponentDialogue* dialogue;	

	dialogue = ecs_component_get(self->ecs, COMPONENT_DIALOGUE, self->dialogue);

	component_dialogue_init
	(
		dialogue,
		self->ecs,
		self->sceneCollection.scenes[self->sceneIndex].string,
		CUTSCENE_DIALOGUE_SPEED
	);
}

static void 
_cutscene_sprite_set(Cutscene* self)
{
	ComponentTextureQuad* textureQuad;	

	textureQuad = ecs_component_get(self->ecs, COMPONENT_TEXTURE_QUAD, self->sprite);

	textureQuad->texture = &self->ecs->resources->textures[self->sceneCollection.scenes[self->sceneIndex].texture];
}

void
cutscene_init(Cutscene* self, ECS* ecs, const SceneCollection sceneCollection)
{
	memset(self, '\0', sizeof(Cutscene));

	self->ecs = ecs;

	self->sceneCollection = sceneCollection;

	self->cursor = entity_cursor_add(self->ecs);

	self->dialogue = entity_dialogue_add
	(
		self->ecs,
		&self->ecs->resources->fonts[FONT_DEFAULT],
		CUTSCENE_BUFFER,
		"",
		(f32*)CUTSCENE_DIALOGUE_POSITION,
		CUTSCENE_DIALOGUE_WRAP,
		CUTSCENE_DIALOGUE_SPEED
	);

	self->sprite = entity_sprite_add
	(
		self->ecs,
		&self->ecs->resources->textures[TEXTURE_DEFAULT],
		CUTSCENE_BUFFER,
		ORIGIN_CENTER,
		(f32*)CUTSCENE_SPRITE_SIZE,
		(f32*)CUTSCENE_SPRITE_POSITION
	);

	_cutscene_dialogue_set(self);
	_cutscene_sprite_set(self);

	entity_text_disappearing_add
	(
	 	ecs,
		&self->ecs->resources->fonts[FONT_SMALL],
		CUTSCENE_BUFFER,
		CUTSCENE_STRING_SKIP,
		CUTSCENE_SKIP_TEXT_POSITION,
		CUTSCENE_SKIP_TEXT_WRAP,
		CUTSCENE_SKIP_TEXT_TIMER
	);

	music_play(&self->ecs->resources->music[MUSIC_CUTSCENE], true);
}

void
cutscene_tick(Cutscene* self)
{
	ComponentDialogue* dialogue;

	if (self->isFinished)
		return;

	dialogue = ecs_component_get(self->ecs, COMPONENT_DIALOGUE, self->dialogue);

	if (control_released(self->ecs->control, CONTROL_ACTION_PRIMARY) && dialogue->isFinished)
	{
		self->sceneIndex++;

		if (self->sceneIndex >= self->sceneCollection.count)
		{
			self->isFinished = true;
			return;
		}
		
		_cutscene_dialogue_set(self);
		_cutscene_sprite_set(self);
	}

	if (control_released(self->ecs->control, CONTROL_ACTION_TERTIARY))
		self->isFinished = true;
}

void
cutscene_free(Cutscene* self)
{



}

