#include "collectible_spawn.h"

#define LEVEL_COLLECTIBLE_COLLECTIBLE_SPAWN_LINE_OFFSET 50.0f

#define COLLECTIBLE_SPAWN_NO_POWER_COUNT 5
#define COLLECTIBLE_SPAWN_ROTTEN_FOOD_NO_POWER_COUNT 6
#define COLLECTIBLE_SPAWN_COUNT 6
#define COLLECTIBLE_SPAWN_ROTTEN_FOOD_COUNT 7
typedef enum CollectibleSpawnType
{
	COLLECTIBLE_SPAWN_FOOD_INDIVIDUAL,
	COLLECTIBLE_SPAWN_FOOD_HORIZONTAL_LINE,
	COLLECTIBLE_SPAWN_FOOD_VERTICAL_LINE,
	COLLECTIBLE_SPAWN_FOOD_CIRCLE,
	COLLECTIBLE_SPAWN_STAMINA_INDIVIDUAL,
	COLLECTIBLE_SPAWN_POWER_INDIVIDUAL,
	COLLECTIBLE_SPAWN_ROTTEN_FOOD_INDIVIDUAL
} CollectibleSpawnType;

typedef struct CollectibleSpawn CollectibleSpawn;

typedef void (*CollectibleSpawnFunction)(CollectibleSpawn*, Level*, vec3);

typedef struct CollectibleSpawn
{
	CollectibleSpawnType spawnType;
	CollectibleType collectibleType;
	CollectibleSpawnFunction function;
	u32 count;
	f32 probability;
} CollectibleSpawn;

void _level_collectible_spawn_individual_add(CollectibleSpawn* self, Level* level, vec3 position);
void _level_collectible_spawn_horizontal_line_add(CollectibleSpawn* self, Level* level, vec3 position);
void _level_collectible_spawn_vertical_line_add(CollectibleSpawn* self, Level* level, vec3 position);
void _level_collectible_spawn_circle_add(CollectibleSpawn* self, Level* level, vec3 position);
void _level_collectible_spawn_refresh_tick(Level* self);
u32 _level_collectible_spawn_food_count_get(Level* self);
void _level_collectible_spawn(Level* self, CollectibleType type, vec3 position);
void _level_collectibles_spawn(Level* self);

static const struct CollectibleSpawn COLLECTIBLE_NO_POWER_SPAWNS[COLLECTIBLE_SPAWN_NO_POWER_COUNT] =
{
	{
		.spawnType = COLLECTIBLE_SPAWN_FOOD_CIRCLE,
		.collectibleType = COLLECTIBLE_FOOD,
		.function = _level_collectible_spawn_circle_add,
		.count = 10,
		.probability = 0.05f
	},
	{
		.spawnType = COLLECTIBLE_SPAWN_FOOD_VERTICAL_LINE,
		.collectibleType = COLLECTIBLE_FOOD,
		.function = _level_collectible_spawn_vertical_line_add,
		.count = 5,
		.probability = 0.10f
	},
	{
		.spawnType = COLLECTIBLE_SPAWN_FOOD_HORIZONTAL_LINE,
		.collectibleType = COLLECTIBLE_FOOD,
		.function = _level_collectible_spawn_horizontal_line_add,
		.count = 5,
		.probability = 0.15f
	},
	{
		.spawnType = COLLECTIBLE_SPAWN_STAMINA_INDIVIDUAL,
		.collectibleType = COLLECTIBLE_STAMINA,
		.function = _level_collectible_spawn_individual_add,
		.count = 1,
		.probability = 0.30f
	},
	{
		.spawnType = COLLECTIBLE_SPAWN_FOOD_INDIVIDUAL,
		.collectibleType = COLLECTIBLE_FOOD,
		.function = _level_collectible_spawn_individual_add,
		.count = 1,
		.probability = 1.0f
	}
};

static const struct CollectibleSpawn COLLECTIBLE_SPAWNS[COLLECTIBLE_SPAWN_COUNT] =
{
	{
		.spawnType = COLLECTIBLE_SPAWN_FOOD_CIRCLE,
		.collectibleType = COLLECTIBLE_FOOD,
		.function = _level_collectible_spawn_circle_add,
		.count = 10,
		.probability = 0.05f
	},
	{
		.spawnType = COLLECTIBLE_SPAWN_FOOD_VERTICAL_LINE,
		.collectibleType = COLLECTIBLE_FOOD,
		.function = _level_collectible_spawn_vertical_line_add,
		.count = 5,
		.probability = 0.10f
	},
	{
		.spawnType = COLLECTIBLE_SPAWN_FOOD_HORIZONTAL_LINE,
		.collectibleType = COLLECTIBLE_FOOD,
		.function = _level_collectible_spawn_horizontal_line_add,
		.count = 5,
		.probability = 0.15f
	},
	{
		.spawnType = COLLECTIBLE_SPAWN_STAMINA_INDIVIDUAL,
		.collectibleType = COLLECTIBLE_STAMINA,
		.function = _level_collectible_spawn_individual_add,
		.count = 1,
		.probability = 0.30f
	},
	{
		.spawnType = COLLECTIBLE_SPAWN_POWER_INDIVIDUAL,
		.collectibleType = COLLECTIBLE_POWER,
		.function = _level_collectible_spawn_individual_add,
		.count = 1,
		.probability = 0.45f
	},
	{
		.spawnType = COLLECTIBLE_SPAWN_FOOD_INDIVIDUAL,
		.collectibleType = COLLECTIBLE_FOOD,
		.function = _level_collectible_spawn_individual_add,
		.count = 1,
		.probability = 1.0f
	}
};

static const struct CollectibleSpawn COLLECTIBLE_ROTTEN_FOOD_NO_POWER_SPAWNS[COLLECTIBLE_SPAWN_ROTTEN_FOOD_NO_POWER_COUNT] =
{
	{
		.spawnType = COLLECTIBLE_SPAWN_FOOD_CIRCLE,
		.collectibleType = COLLECTIBLE_FOOD,
		.function = _level_collectible_spawn_circle_add,
		.count = 10,
		.probability = 0.05f
	},
	{
		.spawnType = COLLECTIBLE_SPAWN_FOOD_VERTICAL_LINE,
		.collectibleType = COLLECTIBLE_FOOD,
		.function = _level_collectible_spawn_vertical_line_add,
		.count = 5,
		.probability = 0.10f
	},
	{
		.spawnType = COLLECTIBLE_SPAWN_FOOD_HORIZONTAL_LINE,
		.collectibleType = COLLECTIBLE_FOOD,
		.function = _level_collectible_spawn_horizontal_line_add,
		.count = 5,
		.probability = 0.15f
	},
	{
		.spawnType = COLLECTIBLE_SPAWN_STAMINA_INDIVIDUAL,
		.collectibleType = COLLECTIBLE_STAMINA,
		.function = _level_collectible_spawn_individual_add,
		.count = 1,
		.probability = 0.30f
	},
	{
		.spawnType = COLLECTIBLE_SPAWN_ROTTEN_FOOD_INDIVIDUAL,
		.collectibleType = COLLECTIBLE_ROTTEN_FOOD,
		.function = _level_collectible_spawn_individual_add,
		.count = 1,
		.probability = 0.45f
	},
	{
		.spawnType = COLLECTIBLE_SPAWN_FOOD_INDIVIDUAL,
		.collectibleType = COLLECTIBLE_FOOD,
		.function = _level_collectible_spawn_individual_add,
		.count = 1,
		.probability = 1.0f
	}
};

static const struct CollectibleSpawn COLLECTIBLE_ROTTEN_FOOD_SPAWNS[COLLECTIBLE_SPAWN_ROTTEN_FOOD_COUNT] =
{
	{
		.spawnType = COLLECTIBLE_SPAWN_FOOD_CIRCLE,
		.collectibleType = COLLECTIBLE_FOOD,
		.function = _level_collectible_spawn_circle_add,
		.count = 10,
		.probability = 0.05f
	},
	{
		.spawnType = COLLECTIBLE_SPAWN_FOOD_VERTICAL_LINE,
		.collectibleType = COLLECTIBLE_FOOD,
		.function = _level_collectible_spawn_vertical_line_add,
		.count = 5,
		.probability = 0.10f
	},
	{
		.spawnType = COLLECTIBLE_SPAWN_FOOD_HORIZONTAL_LINE,
		.collectibleType = COLLECTIBLE_FOOD,
		.function = _level_collectible_spawn_horizontal_line_add,
		.count = 5,
		.probability = 0.15f
	},
	{
		.spawnType = COLLECTIBLE_SPAWN_STAMINA_INDIVIDUAL,
		.collectibleType = COLLECTIBLE_STAMINA,
		.function = _level_collectible_spawn_individual_add,
		.count = 1,
		.probability = 0.30f
	},
	{
		.spawnType = COLLECTIBLE_SPAWN_POWER_INDIVIDUAL,
		.collectibleType = COLLECTIBLE_POWER,
		.function = _level_collectible_spawn_individual_add,
		.count = 1,
		.probability = 0.45f
	},
	{
		.spawnType = COLLECTIBLE_SPAWN_ROTTEN_FOOD_INDIVIDUAL,
		.collectibleType = COLLECTIBLE_ROTTEN_FOOD,
		.function = _level_collectible_spawn_individual_add,
		.count = 1,
		.probability = 0.60f
	},
	{
		.spawnType = COLLECTIBLE_SPAWN_FOOD_INDIVIDUAL,
		.collectibleType = COLLECTIBLE_FOOD,
		.function = _level_collectible_spawn_individual_add,
		.count = 1,
		.probability = 1.0f
	}
};

void
level_collectible_spawn_clear(Level* self)
{
	for (s32 i = 0; i < (s32)self->ecs->lists[COMPONENT_COLLECTIBLE].components.count; i++)
	{
		ComponentCollectible* collectible;
		ComponentPhysics* physics;
		vec3 smokePosition;

		collectible = (ComponentCollectible*)vector_get(&self->ecs->lists[COMPONENT_COLLECTIBLE].components, i);

		physics = ecs_component_get(self->ecs, COMPONENT_PHYSICS, collectible->component.id);

		glm_vec3_copy(physics->position, smokePosition);

		smokePosition[2] += ENTITY_COLLECTIBLE_SMOKE_Z_OFFSET;

		entity_smoke_add
		(
			self->ecs,
			smokePosition
		);

		ecs_entity_delete(self->ecs, collectible->component.id);

		i--;
	}

	sound_play(&self->ecs->resources->sounds[SOUND_APPEAR]);
}

void
_level_collectible_spawn(Level* self, CollectibleType type, vec3 position)
{
	switch (type)
	{
		case COLLECTIBLE_ROTTEN_FOOD:
			entity_collectible_rotten_food_add
			(
				self->ecs,
				position
			);
			break;
		case COLLECTIBLE_FOOD:
			entity_collectible_food_add
			(
				self->ecs,
				position
			);
			break;
		case COLLECTIBLE_STAMINA:
			entity_collectible_stamina_add
			(
				self->ecs,
				position
			);
			break;
		case COLLECTIBLE_POWER:
			entity_collectible_power_add
			(
				self->ecs,
				position
			);
		default:
			break;
	}
}

void
_level_collectible_spawn_individual_add(CollectibleSpawn* self, Level* level, vec3 position)
{
	_level_collectible_spawn(level, self->collectibleType, position);
}

void
_level_collectible_spawn_horizontal_line_add(CollectibleSpawn* self, Level* level, vec3 position)
{
	vec3 collectiblePosition;
	f32 size;

	glm_vec3_copy(position, collectiblePosition);

	size = (ENTITY_COLLECTIBLE_SIZE[0] * self->count);

	collectiblePosition[0] -= size;

	collectiblePosition[0] = CLAMP(collectiblePosition[0], LEVEL_COLLECTIBLE_SPAWN_BOUNDS[0] + size, LEVEL_COLLECTIBLE_SPAWN_BOUNDS[2] - (size * 2));

	for (s32 i = 0; i < (s32)self->count; i++)
	{
		collectiblePosition[0] += ENTITY_COLLECTIBLE_SIZE[0];
		_level_collectible_spawn(level, self->collectibleType, collectiblePosition);
	}
}

void
_level_collectible_spawn_vertical_line_add(CollectibleSpawn* self, Level* level, vec3 position)
{
	vec3 collectiblePosition;
	f32 size;

	glm_vec3_copy(position, collectiblePosition);

	size = (ENTITY_COLLECTIBLE_SIZE[1] * self->count);

	collectiblePosition[1] -= size;

	collectiblePosition[1] = CLAMP(collectiblePosition[1], LEVEL_COLLECTIBLE_SPAWN_BOUNDS[1] + size, LEVEL_COLLECTIBLE_SPAWN_BOUNDS[3] - (size * 2));

	for (s32 i = 0; i < (s32)self->count; i++)
	{
		collectiblePosition[1] += ENTITY_COLLECTIBLE_SIZE[0];
		_level_collectible_spawn(level, self->collectibleType, collectiblePosition);
	}
}

void
_level_collectible_spawn_circle_add(CollectibleSpawn* self, Level* level, vec3 position)
{
	vec3 collectiblePosition;
	f32 angle;
	f32 size;

	angle = 0.0f;

	glm_vec3_copy(position, collectiblePosition);

	size = ENTITY_COLLECTIBLE_SIZE[0] * 2;

	collectiblePosition[0] = CLAMP(collectiblePosition[0], LEVEL_COLLECTIBLE_SPAWN_BOUNDS[0] + (size * 3), LEVEL_COLLECTIBLE_SPAWN_BOUNDS[2] - (size * 3));
	collectiblePosition[1] = CLAMP(collectiblePosition[1], LEVEL_COLLECTIBLE_SPAWN_BOUNDS[1] + (size * 3), LEVEL_COLLECTIBLE_SPAWN_BOUNDS[3] - (size * 3));

	for (s32 i = 0; i < (s32)self->count; i++)
	{
		angle = ((f32)i / self->count) * TAU;

		collectiblePosition[0] += size * cos(angle);
		collectiblePosition[1] += size * sin(angle);

		_level_collectible_spawn(level, self->collectibleType, collectiblePosition);
	}
}

u32
_level_collectible_spawn_food_count_get(Level* self)
{
	u32 count;

	count = 0;

	for (s32 i = 0; i < (s32)self->ecs->lists[COMPONENT_COLLECTIBLE].components.count; i++)
	{
		ComponentCollectible* collectible;

		collectible = (ComponentCollectible*)vector_get(&self->ecs->lists[COMPONENT_COLLECTIBLE].components, i);

		if (collectible->type == COLLECTIBLE_FOOD)
			count++;
	}

	return count;
}

void
_level_collectible_spawn_refresh_tick(Level* self)
{
	u32 foodCount;

	foodCount = _level_collectible_spawn_food_count_get(self);

	if (foodCount == 0)
		_level_collectibles_spawn(self);
}

void
_level_collectibles_spawn(Level* self)
{
	level_collectible_spawn_clear(self);

	Vector spawns;
	u32 collectibleCount;

	vector_init(&spawns, sizeof(u32));

	random_seed_set(self->collectibleSeed);

	collectibleCount = 0;

	while (collectibleCount <= self->settings.collectibleMax)
	{
		f32 random;
		u32 index;

		random = random_f32();

		if (self->settings.isPower && self->isRottenFood)
		{
			for (s32 i = 0; i < COLLECTIBLE_SPAWN_ROTTEN_FOOD_COUNT; i++)
			{
				CollectibleSpawn spawn;

				spawn = COLLECTIBLE_ROTTEN_FOOD_SPAWNS[i];

				if (random < spawn.probability)
				{
					index = i;
					vector_push(&spawns, &index);
					collectibleCount += COLLECTIBLE_ROTTEN_FOOD_SPAWNS[index].count;
					break;
				}
			}
		}
		else if (self->settings.isPower)
		{
			for (s32 i = 0; i < COLLECTIBLE_SPAWN_COUNT; i++)
			{
				CollectibleSpawn spawn;

				spawn = COLLECTIBLE_SPAWNS[i];

				if (random < spawn.probability)
				{
					index = i;
					vector_push(&spawns, &index);
					collectibleCount += COLLECTIBLE_SPAWNS[index].count;
					break;
				}
			}
		}
		else if (self->isRottenFood)
		{
			for (s32 i = 0; i < COLLECTIBLE_SPAWN_ROTTEN_FOOD_NO_POWER_COUNT; i++)
			{
				CollectibleSpawn spawn;

				spawn = COLLECTIBLE_ROTTEN_FOOD_NO_POWER_SPAWNS[i];

				if (random < spawn.probability)
				{
					index = i;
					vector_push(&spawns, &index);
					collectibleCount += COLLECTIBLE_ROTTEN_FOOD_NO_POWER_SPAWNS[index].count;
					break;
				}
			}
		}
		else
		{
			for (s32 i = 0; i < COLLECTIBLE_SPAWN_NO_POWER_COUNT; i++)
			{
				CollectibleSpawn spawn;

				spawn = COLLECTIBLE_NO_POWER_SPAWNS[i];

				if (random < spawn.probability)
				{
					index = i;
					vector_push(&spawns, &index);
					collectibleCount += COLLECTIBLE_NO_POWER_SPAWNS[index].count;
					break;
				}
			}
		}
	};

	for (s32 i = 0; i < (s32)spawns.count; i++)
	{
		u32* index;
		vec3 position;
		bool isInRectangle;

		glm_vec3_zero(position);

		index = (u32*)vector_get(&spawns, i);

		isInRectangle = true;

		while(isInRectangle)
		{
			vec2 testPosition;

			position[0] = random_f32_in_range(LEVEL_COLLECTIBLE_SPAWN_BOUNDS[0], LEVEL_COLLECTIBLE_SPAWN_BOUNDS[2]);
			position[1] = random_f32_in_range(LEVEL_COLLECTIBLE_SPAWN_BOUNDS[1], LEVEL_COLLECTIBLE_SPAWN_BOUNDS[3]);

			testPosition[0] = position[0];
			testPosition[1] = position[1];

			isInRectangle = rectangle_has_point(&LEVEL_COLLECTIBLE_SPAWN_NO_NO_RECTANGLE, testPosition);
		}

		if (self->settings.isPower && self->isRottenFood)
			COLLECTIBLE_ROTTEN_FOOD_SPAWNS[*index].function(&COLLECTIBLE_ROTTEN_FOOD_SPAWNS[*index], self, position);
		else if (self->settings.isPower)
			COLLECTIBLE_SPAWNS[*index].function(&COLLECTIBLE_SPAWNS[*index], self, position);
		else if (self->isRottenFood)
			COLLECTIBLE_ROTTEN_FOOD_NO_POWER_SPAWNS[*index].function(&COLLECTIBLE_ROTTEN_FOOD_NO_POWER_SPAWNS[*index], self, position);
		else
			COLLECTIBLE_NO_POWER_SPAWNS[*index].function(&COLLECTIBLE_NO_POWER_SPAWNS[*index], self, position);
	}

	vector_free(&spawns);

	sound_play(&self->ecs->resources->sounds[SOUND_APPEAR]);

	self->collectibleSeed++;
}

void 
level_collectible_spawn_init(Level* self)
{
	_level_collectibles_spawn(self);
}

void 
level_collectible_spawn_tick(Level* self)
{
	if (self->isCountdownDone && !self->isTimerDone)
		_level_collectible_spawn_refresh_tick(self);
}
