#include "title.h"

void _title_entity_init(Title* self, ECS* ecs);
void _title_level_data_text_init(Title* self, ECS* ecs, u32 level);
void _title_level_select_button_init(Title* self, ECS* ecs, u32 level);

void
_title_level_data_text_init(Title* self, ECS* ecs, u32 level)
{
	const char string[TITLE_LEVEL_DATA_TEXT_MAX];
	LevelData* levelData;
	vec3 position;

	levelData = &self->levelData[level];

	glm_vec3_copy(TITLE_LEVEL_DATA_TEXT_POSITION, position);

	position[0] += TITLE_LEVEL_DATA_TEXT_X_OFFSET * level;

	if (levelData->time <= 0)
	{
		snprintf
		(
			string,
			TITLE_LEVEL_DATA_TEXT_MAX,
			"%s",
			STRING_TITLE_LEVEL_DATA_UNINITIALIZED
		);
	}
	else
	{
		u32 minutes;
		u32 seconds;
		u32 milliseconds;

		time_formatted_get(levelData->time, &minutes, &seconds, &milliseconds);

		snprintf
		(
			string,
			TITLE_LEVEL_DATA_TEXT_MAX,
			TITLE_LEVEL_DATA_TIME_FORMAT,
			minutes,
			seconds,
			milliseconds,
			levelData->score
		);
	}

	entity_text_add
	(
	 	ecs,
		&ecs->resources->fonts[FONT_SMALL],
		RENDERER_BUFFER_UI,
		string,
		position,
		-1
	);
}


void
_title_level_select_button_init(Title* self, ECS* ecs, u32 level)
{
	EntityID id;
	vec3 position;

	glm_vec3_copy(TITLE_LEVEL_SELECT_BUTTON_POSITION, position);

	position[0] += TITLE_LEVEL_SELECT_BUTTON_X_OFFSET * level;

	id = entity_level_select_button_add
	(
		ecs,
		position,
		self->levelData[level].status,
		level
	);

	vector_push(&self->levelSelectButtons, &id);
}

void
_title_entity_init(Title* self, ECS* ecs)
{
	self->cursor = entity_cursor_add(ecs);

	for (s32 i = 0; i < (s32)self->levelCount; i++)
	{
		_title_level_select_button_init(self, ecs, i);
		_title_level_data_text_init(self, ecs, i);
	}

	self->credits = entity_text_add
	(
		ecs,
		&ecs->resources->fonts[FONT_DEFAULT], 
		RENDERER_BUFFER_UI,
		STRING_TITLE_LEVEL_SELECT,
		TITLE_LEVEL_SELECT_POSITION,
		-1
	);

	self->levelSelectText = entity_text_add
	(
		ecs,
		&ecs->resources->fonts[FONT_SMALL], 
		RENDERER_BUFFER_UI,
		STRING_TITLE_CREDITS,
		TITLE_CREDITS_POSITION,
		-1
	);

	self->bg = entity_sprite_add
	(
		ecs, 
		&ecs->resources->textures[TEXTURE_TITLE_BG],
		RENDERER_BUFFER_BACKGROUND, 
		ORIGIN_TOP_LEFT,
		TITLE_BG_SIZE,
		TITLE_BG_POSITION
	);

	self->logo = entity_logo_add
	(
		ecs, 
		TITLE_LOGO_POSITION
	);
}

void
title_init(Title* self, ECS* ecs, LevelData* levelData, u32 levelCount)
{
	memset(self, '\0', sizeof(Title));

	self->ecs = ecs;
	self->levelCount = levelCount;

	self->levelData = malloc(sizeof(LevelData) * levelCount);
	memcpy(self->levelData, levelData, sizeof(LevelData) * levelCount);
	
	vector_init(&self->levelSelectButtons, sizeof(EntityID));

	_title_entity_init(self, ecs);

	music_play(&self->ecs->resources->music[MUSIC_TITLE], false);
}

void
title_free(Title* self)
{
	free(self->levelData);
	vector_clear(&self->levelSelectButtons);
}

void
title_tick(Title* self)
{
	for (s32 i = 0; i < (s32)self->levelSelectButtons.count; i++)
	{
		ComponentButton* button;
		EntityID* id;

		id = vector_get(&self->levelSelectButtons, i);

		button = ecs_component_get(self->ecs, COMPONENT_BUTTON, *id);

		if (button->isJustPressed)
		{
			self->levelSelected = i;
			self->isLevelSelected = true;
		}
	}

}
